@@ -1,18 +1,25 @@
 #ifndef R2_TYPES_BASE_H
 #define R2_TYPES_BASE_H
 
-#include <ctype.h>
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-#define cut8 const unsigned char
-#define ut64 unsigned long long
-#define st64 long long
-#define ut32 unsigned int
-#define st32 int
-#define ut16 unsigned short
-#define st16 short
-#define ut8 unsigned char
-#define st8 signed char
-#define boolt int
+#include <ctype.h>
+#include <sys/types.h>
+#include <limits.h>
+#include <stdint.h>
+#include <stdbool.h>
+
+#define cut8 const uint8_t
+#define ut64 uint64_t
+#define st64 int64_t
+#define ut32 uint32_t
+#define st32 int32_t
+#define ut16 uint16_t
+#define st16 int16_t
+#define ut8 uint8_t
+#define st8 int8_t
 
 #if defined(_MSC_VER)
 # define R_ALIGNED(x) __declspec(align(x))
@@ -20,6 +27,8 @@
 # define R_ALIGNED(x) __attribute__((aligned(x)))
 #endif
 
+#define R_IGNORE_RETURN(x) if ((x)) {;}
+
 typedef R_ALIGNED(1) ut16 uut16;
 typedef R_ALIGNED(1) ut32 uut32;
 typedef R_ALIGNED(1) ut64 uut64;
@@ -50,7 +59,7 @@ typedef struct _ut256 {
 	ut128 Low;
 	ut128 High;
 } ut256;
-typedef struct _utX{
+typedef struct _utX {
 	ut80 v80;
 	ut96 v96;
 	ut128 v128;
@@ -59,9 +68,6 @@ typedef struct _utX{
 
 #include <stdbool.h>
 
-#define R_EMPTY { 0 }
-#define R_EMPTY2 {{ 0 }}
-
 /* limits */
 #undef UT64_MAX
 #undef UT64_GT0
@@ -69,16 +75,19 @@ typedef struct _utX{
 #undef UT64_MIN
 #undef UT32_MAX
 #undef UT32_MIN
-#define ST64_MAX 0x7FFFFFFFFFFFFFFFULL
-#define ST64_MIN (-ST64_MAX-1)
-#define UT64_MAX 0xFFFFFFFFFFFFFFFFULL
-#define UT64_GT0 0x8000000000000000ULL
-#define UT64_LT0 0x7FFFFFFFFFFFFFFFULL
+#undef UT16_MIN
+#undef UT8_MIN
+#define ST64_MAX ((st64)0x7FFFFFFFFFFFFFFFULL)
+#define ST64_MIN ((st64)(-ST64_MAX-1))
+#define UT64_MAX ((ut64)0xFFFFFFFFFFFFFFFFULL)
+#define UT64_GT0 ((ut64)0x8000000000000000ULL)
+#define UT64_LT0 ((ut64)0x7FFFFFFFFFFFFFFFULL)
 #define UT64_MIN 0ULL
 #define UT64_32U 0xFFFFFFFF00000000ULL
 #define UT64_16U 0xFFFFFFFFFFFF0000ULL
 #define UT64_8U  0xFFFFFFFFFFFFFF00ULL
 #define UT32_MIN 0U
+#define UT16_MIN 0U
 #define UT32_GT0 0x80000000U
 #define UT32_LT0 0x7FFFFFFFU
 #define ST32_MAX 0x7FFFFFFF
@@ -90,10 +99,24 @@ typedef struct _utX{
 #define UT16_GT0 0x8000U
 #define UT16_MAX 0xFFFFU
 #define ST8_MAX  0x7F
-#define ST8_MIN  (-ST8_MAX-1)
+#define ST8_MIN  (-ST8_MAX - 1)
 #define UT8_GT0  0x80U
 #define UT8_MAX  0xFFU
 #define UT8_MIN  0x00U
+#define ASCII_MIN 32
+#define ASCII_MAX 127
+
+#if SSIZE_MAX == ST32_MAX
+#define SZT_MAX  UT32_MAX
+#define SZT_MIN  UT32_MIN
+#define SSZT_MAX  ST32_MAX
+#define SSZT_MIN  ST32_MIN
+#else
+#define SZT_MAX  UT64_MAX
+#define SZT_MIN  UT64_MIN
+#define SSZT_MAX  ST64_MAX
+#define SSZT_MIN  ST64_MIN
+#endif
 
 #define UT64_ALIGN(x) (x + (x - (x % sizeof (ut64))))
 #define UT32_ALIGN(x) (x + (x - (x % sizeof (ut32))))
@@ -102,23 +125,46 @@ typedef struct _utX{
 #define UT32_LO(x) ((ut32)((x)&UT32_MAX))
 #define UT32_HI(x) ((ut32)(((ut64)(x))>>32)&UT32_MAX)
 
-/* preventive math overflow checks */
-#if !defined(SZT_ADD_OVFCHK)
-#define SZT_ADD_OVFCHK(x,y) ((SIZE_MAX - (x)) < (y))
+#define R_BETWEEN(x,y,z) (((y)>=(x)) && ((y)<=(z)))
+#define R_ROUND(x,y) ((x)%(y))?(x)+((y)-((x)%(y))):(x)
+#define R_DIM(x,y,z) (((x)<(y))?(y):((x)>(z))?(z):(x))
+#ifndef R_MAX_DEFINED
+#define R_MAX(x,y) (((x)>(y))?(x):(y))
+#define R_MAX_DEFINED
 #endif
-#define UT64_ADD_OVFCHK(x,y) ((UT64_MAX - (x)) < (y))
-#define UT32_ADD_OVFCHK(x,y) ((UT32_MAX - (x)) < (y))
-#define UT16_ADD_OVFCHK(x,y) ((UT16_MAX - (x)) < (y))
-#define UT8_ADD_OVFCHK(x,y) ((UT8_MAX - (x)) < (y))
+#ifndef R_MIN_DEFINED
+#define R_MIN(x,y) (((x)>(y))?(y):(x))
+#define R_MIN_DEFINED
+#endif
+#define R_ABS(x) (((x)<0)?-(x):(x))
+#define R_BTW(x,y,z) (((x)>=(y))&&((y)<=(z)))?y:x
+
+#include "r_types_overflow.h"
 
 /* copied from bithacks.h */
-#define B_IS_SET(x, n)   (((x) & (1ULL<<(n)))?1:0)
-#define B_SET(x, n)      ((x) |= (1ULL<<(n)))
-#define B_EVEN(x)        (((x)&1)==0)
+#define B_IS_SET(x, n)   (((x) & (1ULL << (n)))? 1: 0)
+#define B_SET(x, n)      ((x) |= (1ULL << (n)))
+#define B_EVEN(x)        (((x) & 1) == 0)
 #define B_ODD(x)         (!B_EVEN((x)))
-#define B_UNSET(x, n)    ((x) &= ~(1ULL<<(n)))
-#define B_TOGGLE(x, n)   ((x) ^= (1ULL<<(n)))
+#define B_UNSET(x, n)    ((x) &= ~(1ULL << (n)))
+#define B_TOGGLE(x, n)   ((x) ^= (1ULL << (n)))
 
+#define B11111 31
+#define B11110 30
+#define B11101 29
+#define B11100 28
+#define B11011 27
+#define B11010 26
+#define B11001 25
+#define B11000 24
+#define B10111 23
+#define B10110 22
+#define B10101 21
+#define B10100 20
+#define B10011 19
+#define B10010 18
+#define B10001 17
+#define B10000 16
 #define B1111 15
 #define B1110 14
 #define B1101 13
@@ -147,6 +193,17 @@ typedef struct _utX{
 #define NAN (0.0f/0.0f)
 #endif
 
+/* A workaround against libc headers redefinition of __attribute__:
+ * Standard include has lines like
+ * #if (GCC_VERSION < 2007)
+ * # define __attribute__(x)
+ * #endif
+ * So we have do remove this define for TinyCC compiler
+ */
+#if defined(__TINYC__) && (GCC_VERSION < 2007)
+#undef __attribute__
+#endif
+
 #ifdef _MSC_VER
 #define R_PACKED( __Declaration__ ) __pragma( pack(push, 1) ) __Declaration__ __pragma( pack(pop) )
 #undef INFINITY
@@ -162,4 +219,14 @@ typedef struct _utX{
 #define DEBUGGER 0
 #endif
 
+#define HEAPTYPE(x) \
+	static x* x##_new(x n) {\
+		x *m = malloc(sizeof (x));\
+		return m? *m = n, m: m; \
+	}
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif // R2_TYPES_BASE_H
